@@ -60,10 +60,16 @@
 #' Fixed values may compromise model fitting, and scaling them is a safe
 #' option to avoid this problem. As a rule of thumb, to our experience,
 #' \code{start = 0.1} generally performs well with {-1, 0, 1} weights.
+#' @param SE If "standard" (default), with \code{algo = "lavaan"},
+#' conventional standard errors are computed based on inverting the observed
+#' information matrix. If "none", no standard errors are computed.
+#' @param n_rep Number of randomization replicates (default = 1000),
+#' for permutation flip or boostrap samples, if \code{algo = "ricf"}.
 #' @param limit An integer value corresponding to the network size
 #' (i.e., number of nodes). Beyond this limit, the execution under
-#' \code{algo = "lavaan"} will be ridirected to \code{algo = "ricf"}, if
-#' fit is either 0 or 1, or to \code{algo = "ggm"}, if \code{fit = 2}.
+#' \code{algo = "lavaan"} will run with \code{SE = "none"}, if 
+#' \code{fit = 0}, or will be ridirected to \code{algo = "ricf"}, if
+#' \code{fit = 1}, or to \code{algo = "ggm"}, if \code{fit = 2}.
 #' This redirection is necessary to reduce the computational demand of
 #' standard error estimation by lavaan. Increasing this number will
 #' enforce lavaan execution when \code{algo = "lavaan"}.
@@ -77,9 +83,7 @@
 #' estimated from directed interactions and residual covariances (psi
 #' coefficients) from bidirected, undirected, or mutual interactions.
 #' If a group variable is given, exogenous group effects on nodes (gamma
-#' coefficients) will be estimated. This will also lead to the estimation
-#' of a set of aggregated group effects, if \code{algo = "ricf"} (see
-#' \code{\link[SEMgraph]{SEMgsa}}).
+#' coefficients) or edges (delta coefficients) will be estimated.
 #' By default, maximum likelihood parameter estimates and P-values for
 #' parameter sets are computed by conventional z-test (= estimate/SE),
 #' and fits it through the \code{\link[lavaan]{lavaan}} function, via
@@ -161,38 +165,24 @@
 #' Larson JL and Owen AB (2015). Moment based gene set tests. BMC
 #' Bioinformatics, 16: 132. <https://doi.org/10.1186/s12859-015-0571-7>
 #'
-#' Palluzzi F, Grassi M (2021). SEMgraph: An R Package for Causal Network
+#' Grassi M, Palluzzi F, Tarantino B (2022). SEMgraph: An R Package for Causal Network
 #' Analysis of High-Throughput Data with Structural Equation Models.
-#' <arXiv:2103.08332>
+#' Bioinformatics, 2022;, btac567, https://doi.org/10.1093/bioinformatics/btac567
 #'
-#' Williams D (2020). GGMncv: Gaussian Graphical Models with Non-Convex
-#' Penalties. R package version 1.1.0.
-#' <https://CRAN.R-project.org/package=GGMncv/>
-#'
-#' @seealso See \code{\link[ggm]{fitAncestralGraph}} for RICF algorithm
-#' details, \code{\link[flip]{flip}} for randomization P-values, and
-#' \code{\link[GGMncv]{constrained}} for constrained GGM, and
-#' \code{\link[GGMncv]{inference}} for de-sparsified P-values.
+#' @seealso See \code{\link[ggm]{fitAncestralGraph}} and \code{\link[ggm]{fitConGraph}}
+#' for RICF algorithm and constrained GGM algorithm details, respectively.
 #'
 #' @examples
 #'
 #' #### Model fitting (no group effect)
 #'
-#' sem0 <- SEMrun(graph = sachs$graph, data = log(sachs$pkc), algo = "lavaan")
+#' sem0 <- SEMrun(graph = sachs$graph, data = log(sachs$pkc))
 #' summary(sem0$fit)
 #' head(parameterEstimates(sem0$fit))
 #'
-#' sem0 <- SEMrun(graph = sachs$graph, data = log(sachs$pkc), algo = "ricf")
-#' summary(sem0$fit)
-#' head(sem0$fit$parameterEstimates)
-#'
-#' sem0 <- SEMrun(graph = sachs$graph, data = log(sachs$pkc), algo = "cggm")
-#' summary(sem0$fit)
-#' head(sem0$fit$parameterEstimates)
-#'
 #' # Graphs
-#' gplot(sem0$graph, main = "edge differences")
-#' plot(sem0$graph, layout = layout.circle, main = "edge differences")
+#' gplot(sem0$graph, main = "significant edge weights")
+#' plot(sem0$graph, layout = layout.circle, main = "significant edge weights")
 #'
 #'
 #' #### Model fitting (common model, group effect on nodes)
@@ -206,8 +196,8 @@
 #' head(parameterEstimates(sem1$fit))
 #'
 #' # Graphs
-#' gplot(sem1$graph, main = "node differences")
-#' plot(sem1$graph, layout = layout.circle, main = "node differences")
+#' gplot(sem1$graph, main = "Between group node differences")
+#' plot(sem1$graph, layout = layout.circle, main = "Between group node differences")
 #'
 #'
 #' #### Two-group model fitting (group effect on edges)
@@ -243,15 +233,14 @@
 #' g <- properties(g2)[[1]]
 #'
 #' # plot graph
-#' library(org.Hs.eg.db)
-#' V(g)$label <- mapIds(org.Hs.eg.db, V(g)$name, 'SYMBOL', 'ENTREZID')
 #' E(g)$color<- E(g2)$color[E(g2) %in% E(g)]
 #' gplot(g, l = "fdp", main="node and edge group differences")
 #'
 #' }
 #'
 SEMrun <- function(graph, data, group = NULL, fit = 0, algo = "lavaan",
-                   start = NULL, limit = 100, ...)
+                   start = NULL, SE = "standard", n_rep = 1000,
+				   limit = 100, ...)
 {
 	if (is.null(group) & fit != 0) fit <- 0
 	if (!is.null(group) & fit == 0) fit <- 1
@@ -259,38 +248,36 @@ SEMrun <- function(graph, data, group = NULL, fit = 0, algo = "lavaan",
 	if (fit == 0) {
 		if (algo == "lavaan") {
 			return(fit = SEMfit(graph = graph, data = data, group = NULL,
-			                    start = start,
-			                    limit = limit))
+			                    start = start, SE = SE, limit = limit))
 		} else if (algo == "cggm") {
 			return(fit = SEMggm(graph = graph, data = data, group = NULL))
 		} else if (algo == "ricf") {
-			return(fit = SEMricf(graph = graph, data = data, group = NULL))
+			return(fit = SEMricf(graph = graph, data = data, group = NULL,
+								 n_rep = n_rep))
 		}
 	}
 
 	if (fit == 1) {
 		if (algo == "lavaan") {
 			return(fit = SEMfit(graph = graph, data = data, group = group,
-			                    start = start,
-			                    limit = limit))
+			                    start = start, SE = SE, limit = limit))
 		} else if (algo == "cggm") {
 			return(fit = SEMggm(graph = graph, data = data, group = group))
 		} else if( algo == "ricf" ) {
 			return(fit = SEMricf(graph = graph, data = data, group = group,
-			                     n_rep = 1000))
+			                     n_rep = n_rep))
 		}
 	}
 
 	if (fit == 2) {
 		if (algo == "lavaan") {
 			return(fit = SEMfit2(graph = graph, data = data, group = group,
-			                     start = start,
-			                     limit = limit))
+			                     start = start, SE = SE, limit = limit))
 		} else if (algo == "cggm") {
 			return(fit = SEMggm2(graph = graph, data = data, group = group))
 		} else if (algo == "ricf") {
 			return(fit = SEMricf2(graph = graph, data = data, group = group,
-			                      n_rep = 0))
+			                      n_rep = n_rep))
 		}
 	}
 }
@@ -402,11 +389,16 @@ SEMstart <- function(ig, data, group, a, ...)
 	}
 }
 
-SEMfit <- function(graph, data, group = NULL, start = NULL, limit = 100,
-                   SE = "standard", ...)
+SEMfit <- function(graph, data, group = NULL, start = NULL, fit = 0,
+					SE = "standard", limit = 100,...)
 {
-	# Model fitting with GGM algo if n.nodes > limit
-	if (vcount(graph) > limit) {
+	# Change SEM fitting if n.nodes > limit
+	if (vcount(graph) > limit & fit == 0) {
+		message("WARNING: very large input graph (>", limit, " nodes) !
+		 SEs are not computed...\n")
+		SE <- "none"
+	}
+	if (vcount(graph) > limit & fit == 1) {
 		message("WARNING: very large input graph (>", limit, " nodes) !
 		 RICF solver activated...\n")
 		return(fit = SEMricf(graph = graph, data = data, group = group))
@@ -477,7 +469,7 @@ SEMfit <- function(graph, data, group = NULL, start = NULL, limit = 100,
 	}
 
 	# Output objects
-	ig <- colorGraph(est = est, graph = ig, group = group, alpha = 0.05)
+	if (SE != "none") ig <- colorGraph(est = est, graph = ig, group = group, alpha = 0.05)
 	if (is.null(group)) dataXY <- cbind(group = rep(NA, n), dataXY)
 	colnames(dataXY) <- gsub("z", "", colnames(dataXY))
 
@@ -485,8 +477,8 @@ SEMfit <- function(graph, data, group = NULL, start = NULL, limit = 100,
 	            dataXY = dataXY))
 }
 
-SEMfit2 <- function(graph, data, group, start = NULL, limit = 100,
-                    SE = "standard", ...)
+SEMfit2 <- function(graph, data, group, start = NULL, SE = "standard",
+                    limit = 100, ...)
 {
 	# Model fitting with GGM algo if n.nodes > limit
 	if (vcount(graph) > limit) {
@@ -577,9 +569,8 @@ SEMfit2 <- function(graph, data, group, start = NULL, limit = 100,
 					  d_lower, d_upper)
 		dest$lhs <- sub("z", "", dest$lhs)
 		dest$rhs <- sub("z", "", dest$rhs)
-		dest <- data.frame(lapply(dest,
-						   function(y) if(is.numeric(y)) round(y, 3) else y))
-		ig <- colorGraph(est = dest, graph = ig, group = NULL, alpha = 0.05)
+		class(dest)<- c("lavaan.data.frame" ,"data.frame")
+		if (SE != "none") ig <- colorGraph(est = dest, graph = ig, group = NULL, alpha = 0.05)
 	} else {
 		dest <- NULL
 	}
@@ -626,34 +617,37 @@ SEMricf<- function (graph, data, group = NULL, random.x = FALSE, n_rep = 1000, .
 		" iterations"), "\n\n")
 	idx <- fitIndices(n, fit$df, covXY, fit$Shat)
 	cat("deviance/df:", idx[1]/idx[2], " srmr:", round(idx[3], 7), "\n\n")
+	
 	est <- parameterEstimates.RICF(fit)
-	if (!is.null(group)) {
-		gest <- gest.RICF(fit = fit, data = dataY, group = group, 
-			n_rep = n_rep)
-		pval1 <- Brown.test(x = dataY, p = gest[[1]][, 4],
-			theta = gest[[1]][, 2], tail = "positive")
-		pval2 <- Brown.test(x = dataY, p = gest[[1]][, 4],
-			theta = gest[[1]][, 2], tail = "negative")
-		cat("Brown's combined P-value of node activation:", 
-			pval1, "\n\n")
-		cat("Brown's combined P-value of node inhibition:", 
-			pval2, "\n\n")
-		ig <- colorGraph(est = gest[[1]], ig, group, alpha = 0.05)
-		pval <- c(pval1, pval2)
-	}else{
-		gest <- list(NULL, NULL)
-		pval <- NULL
+	gest <- list(NULL, NULL)
+	pval <- NULL
+	
+	if (!is.null(group) & n_rep != 0) {
+	 gest <- flip.RICF(fit = fit, data = dataY, group = group, n_rep = n_rep)
+	 pval1 <- Brown.test(x = dataY, p = gest[[1]][,4],
+		theta = gest[[1]][,2], tail = "positive")
+	 pval2 <- Brown.test(x = dataY, p = gest[[1]][,4],
+		theta = gest[[1]][,2], tail = "negative")
+	 cat("Brown's combined P-value of node activation:", pval1, "\n\n")
+	 cat("Brown's combined P-value of node inhibition:", pval2, "\n\n")
+	 ig <- colorGraph(est = gest[[1]], ig, group, alpha = 0.05)
+	 pval <- c(pval1, pval2)
 	}
+	
+	if (is.null(group) & n_rep != 0) {
+	 est <- boot.RICF(A, dataXY, group, est, n_rep)
+	 ig <- colorGraph(est, graph=ig, group=group, alpha=0.05)
+	 dataXY <- cbind(group = rep(NA, n), dataXY)
+	}
+	
 	fit <- list(ricf = fit, fitIdx = idx, parameterEstimates = est)
-	if (is.null(group)) {
-		dataXY <- cbind(group = rep(NA, n), dataXY)
-	}
 	class(fit) <- "RICF"
+	
 	return(list(fit = fit, gest = gest[[1]], model = NULL, graph = ig, 
 		dataXY = dataXY, r_gest = gest[[2]], pval = pval))
 }
 
-SEMricf2 <- function(graph, data, group, random.x = FALSE, n_rep = 0, ...)
+SEMricf2 <- function(graph, data, group, random.x = FALSE, n_rep = 1000, ...)
 {
 	# Set graph and data objects
 	nodes <- colnames(data)[colnames(data) %in% V(graph)$name]
@@ -666,12 +660,12 @@ SEMricf2 <- function(graph, data, group, random.x = FALSE, n_rep = 0, ...)
 	n <- n1 + n0
 
 	# Fitting RICF for group = 1
-	fit1 <- quiet(SEMricf(graph, data1, group = NULL, random.x = FALSE, n_rep))
-	est1 <- fit1$fit$parameterEstimates$Reg
+	fit1 <- quiet(SEMricf(graph, data1, group = NULL, random.x = FALSE, n_rep = 0))
+	est1 <- fit1$fit$parameterEstimates
 
 	# Fitting RICF for group = 0
-	fit0 <- quiet(SEMricf(graph, data0, group = NULL, random.x = FALSE, n_rep))
-	est0 <- fit0$fit$parameterEstimates$Reg
+	fit0 <- quiet(SEMricf(graph, data0, group = NULL, random.x = FALSE, n_rep = 0))
+	est0 <- fit0$fit$parameterEstimates
 
 	# Two-group fit indices
 	it <- fit1$fit$ricf$it + fit0$fit$ricf$it
@@ -681,19 +675,29 @@ SEMricf2 <- function(graph, data, group, random.x = FALSE, n_rep = 0, ...)
 	df <- fit1$fit$fitIdx[2] + fit0$fit$fitIdx[2]
 	cat("deviance/df:", dev/df , " srmr:", round(srmr, 7), "\n\n")
 
-	# Output objects
-	est <- list(Group_1 = est1, Group_0 = est0)
-	d_est <- est[[1]]$est - est[[2]]$est
-	dest <- cbind(est[[1]][,c(1:3)], d_est)
-	fit <- list(Group_0=fit0[[1]], Group_1 = fit1[[1]],
-	            parameterEstimates = est)
-	dataXY<- cbind(c(rep(1, n1), rep(0, n0)), rbind(data1, data0))
+	# edge differences based on group randomization with n_rep=1000
+	d_est<- est1$est - est0$est
+	dest<- cbind(est1[,c(1:3)], d_est)[est1$op == "~",]
+	
+	if (n_rep != 0) {
+	 A<- ifelse(abs(fit1$fit$ricf$Bhat) > 0, 1, 0)
+	 diag(A)<- 0
+	 dest<- boot.RICF(t(A), dataY, group, dest, R=n_rep)
+	 pval1<- Brown.test(x=dataY, p=dest$pvalue, theta=dest$d_est, tail="positive")
+	 pval2<- Brown.test(x=dataY, p=dest$pvalue, theta=dest$d_est, tail="negative")
+	 cat("Brown's combined P-value of edge activation:", pval1, "\n\n")
+     cat("Brown's combined P-value of edge inhibition:", pval2, "\n\n")
+	 ig<- colorGraph(est=dest, graph=ig, group=NULL, alpha=0.05)
+	}
 
-	return(list(fit = fit, dest = dest, model = NULL, graph = ig,
-	            dataXY = dataXY))
+	# output objects:
+	fit<- list(Group_1 = fit1[[1]], Group_0 = fit0[[1]])	
+	dataXY<- cbind(c(rep(1,n1),rep(0,n0)),rbind(data1,data0))
+		
+	return(list(fit = fit, dest = dest, model = NULL, graph = ig, dataXY = dataXY))
 }
 
-gest.RICF<- function(fit, data, group, n_rep, ...)
+flip.RICF<- function(fit, data, group, n_rep, ...)
 {
 	# Permutation pvalues of group -> nodes:
 	p <- ncol(data)
@@ -714,14 +718,51 @@ gest.RICF<- function(fit, data, group, n_rep, ...)
 	return( list(gest=gest@res, r_gest=gest@permT) )
 }
 
+boot.RICF <- function(A, Z, group, L, R, ...)
+{
+	est<- function(Z, i){
+	 Z <- Z[i,]
+	 if (corpcor::is.positive.definite(cor(Z))) {
+	  covXY <- cor(Z)
+	 }else{
+	  covXY<- corpcor::cor.shrink(Z, verbose = FALSE)
+	 }
+	 fit <- ggm::fitAncestralGraph(A, covXY, n=nrow(Z), tol = 1e-06)
+	 B <- parameterEstimates.RICF(fit)$est
+	}
+	dest<- function(Z, i){
+	 Zi<- Z[i,]
+	 D <- est(Zi[group == 1,]) - est(Zi[group == 0,])
+	}
+
+	message(paste0("Model randomization with B = ", R, " bootstrap samples ...\n\n"))
+	ncpus<- parallel::detectCores(logical = FALSE)
+	if (is.null(group)){
+	 xboot<- boot::boot(Z, est, R, parallel="snow", ncpus=ncpus)
+	 #xboot<- boot::boot(Z, FUN1, R=R)
+	}else{
+	 xboot<- boot::boot(Z, dest, strata=group, R=R, parallel="snow", ncpus=ncpus)
+	 #xboot<- boot::boot(Z, est, strata=group, R=R)
+	}
+	t0<- xboot$t0[1:nrow(L)]
+	se<- apply(xboot$t[,1:nrow(L)], MARGIN=2, sd)
+	z <- t0/se
+	
+	est<- data.frame(L, se = se, z=z,
+	 pvalue = 2*(1-pnorm(abs(z))),
+	 ci.lower = (t0-1.96*se),
+	 ci.upper = (t0+1.96*se))
+	class(est)<- c("lavaan.data.frame" ,"data.frame")
+
+	return(est)
+}
+
 parameterEstimates.RICF <- function(object, ...)
 {
-	# Output of parameter estimates
-
+	# Convert into a vector the output of parameter estimates
 	p <- nrow(object$Bhat)
 	B <- gdata::unmatrix(diag(p) - object$Bhat, byrow = FALSE)
 	B <- B[which(B != 0)]
-
 	O <- ifelse(lower.tri(object$Ohat, diag = TRUE), object$Ohat, 0)
 	diag(O) <- ifelse(diag(O) == 0, 1, diag(O))
 	rownames(O) <- colnames(O) <- rownames(object$Ohat)
@@ -742,357 +783,186 @@ parameterEstimates.RICF <- function(object, ...)
 			             stringsAsFactors = FALSE))
 		}
 	}
-
-	rownames(est) <- NULL
-	reg <- est[which(est$op == "~"),]
-	cov <- est[which(est$op == "~~"),]
-	sel <- which(cov$lhs == cov$rhs)
-	var <- cov[sel,]
-	cov <- cov[-sel,]
-
-	return(list(Reg = reg, Cov = cov, Var = var))
+	rownames(est)<- NULL
+	
+	return(est)
 }
 
-#' @title RICF model summary
-#'
-#' @description Generate a summary for a RICF solver similar to
-#' lavaan-formatted summary
-#'
-#' @param object A RICF fitted model object.
-#' @param ... Currently ignored.
-#'
-#' @method summary RICF
-#' @return Shown the lavaan-formatted summary to console
-#'
-#' @author Mario Grassi \email{mario.grassi@unipv.it}
-#'
-#' @seealso \code{\link[SEMgraph]{SEMrun}}.
-#'
-#' @examples
-#' sem0 <- SEMrun(sachs$graph, log(sachs$pkc), algo = "ricf")
-#' summary(sem0$fit)
-#'
-#' @export
-summary.RICF <- function(object, ...)
+SEMggm<- function(graph, data, group = NULL, method = "none", alpha = 0.05, ...) 
 {
-	.local <- function(object) {
-		it <- object$ricf$it
-		t <- object$fitIdx[6]
-		n <- object$fitIdx[5]
-		dev <- round(object$fitIdx[1], 3)
-		df <- object$fitIdx[2]
-		srmr <- round(object$fitIdx[3], 3)
-
-		cat(paste0("RICF solver ended normally after ", it, " iterations"),
-		           "\n\n")
-		cat(paste0("  Estimator                                       ML"),
-		           "\n")
-		cat(paste0("  Optimization method                             RICF"),
-		           "\n\n")
-		cat(paste0("  Number of free parameters                       ",
-		           t), "\n\n")
-		cat(paste0("  Number of observations                          ",
-		           n), "\n")
-		cat("\nModel Test User Model\n\n")
-		cat(paste0("  Test statistic (Deviance)                       ",
-		           dev), "\n")
-		cat(paste0("  Degrees of freedom (df)                         ",
-		           df), "\n")
-		cat(paste0("  Deviance/df                                     ",
-		           round(dev/df, 3)), "\n")
-		cat(paste0("  Standardized Root Mean Square Residual (srmr)   ",
-		           srmr), "\n")
-		cat("\nParameter Estimates:\n\n")
-
-		#print(fit$parameterEstimates)
-		K <- c("Regressions:", "Covariances:", "Variances:")
-		L <- object$parameterEstimates
-		for (l in 1:3) {
-			cat(K[l], "\n\n")
-			print(data.frame(lapply(L[[l]], function(y) {
-					if(is.numeric(y)) round(y, 3) else y
-				}
-			)))
-			cat("\n")
-		}
-	}
-	.local(object)
-}
-
-SEMggm <- function(graph, data, group = NULL, method = "none",
-                   alpha = 0.05, ...)
-{
-	# Set data objects
-	nodes <- colnames(data)[colnames(data) %in% V(graph)$name]
-	dataY <- data[,nodes]
-	if (is.null(group)) {
-		dataXY <- dataY
-	} else {
-		dataXY <- cbind(group, dataY)
-	}
-	n <- nrow(dataXY)
-	p <- ncol(dataXY)
-	if (corpcor::is.positive.definite(cor(dataXY)[1:p, 1:p])) {
-	 covXY <- cor(dataXY)[1:p, 1:p]
-	} else {
+	# Set data objects :
+	nodes<- colnames(data)[colnames(data) %in% V(graph)$name]
+	dataY<- data[,nodes]
+	if (is.null(group)){
+     dataXY <- dataY
+    }else{ dataXY <- cbind(group, dataY)}
+	n<- nrow(dataXY)
+	p<- ncol(dataXY)
+	if( corpcor::is.positive.definite(cor(dataXY)[1:p,1:p]) ){
+	 covXY<- cor(dataXY)[1:p,1:p]
+	}else{ 
 	 covXY<- corpcor::cor.shrink(dataXY,verbose=TRUE)
 	 if (attributes(covXY)$lambda > 0.5) message(
 	  "WARNING: lambda > 0.5, correlation matrix is 'shrinked' near the identity matrix !\n")
 	 covXY<- covXY[1:p,1:p]
 	}
 
-	# Set graph objects
-	ig <- induced_subgraph(graph, vids = which(V(graph)$name %in% nodes))
-	adj <- as_adj(as.undirected(ig), sparse = FALSE)[nodes, nodes]
-	E(ig)$weight <- ifelse(which_mutual(ig), 100, 1)
-	dadj <- as_adj(ig, attr = "weight", sparse = FALSE)[nodes, nodes]
+	# Set graph objects :
+	ig<- induced_subgraph(graph, vids= which(V(graph)$name %in% nodes))
+	E(ig)$weight<- ifelse(which_mutual(ig), 100, 1)
+	dadj<- as_adj(ig, attr="weight", sparse=FALSE)[nodes,nodes]
+	diag(dadj) <- 100
+	adj<- as_adj(as.undirected(ig), sparse=FALSE)[nodes,nodes]
+	if( !is.null(group) ){
+	 adj<- cbind(c(0,rep(1, p-1)), rbind(rep(1, p-1),adj))
+	 colnames(adj)[1]<- rownames(adj)[1]<- "group"
+	 dadj<- cbind(rep(0, p), rbind(rep(1, p-1),dadj))
+	 colnames(dadj)[1]<- rownames(dadj)[1]<- "group"
+	}
+	
+	# constrained GGM 
+	cggm<- ggm::fitConGraph(amat=adj, S=covXY, n=n)
+	Sigma<- cggm$Shat
+	Theta<- solve(cggm$Shat)
+	#cggm<- GGMncv::constrained(covXY, adj)
+	#Sigma<- cggm$Sigma
+	#Theta<- cggm$Theta
+	rownames(Sigma)<- colnames(Sigma) <- colnames(dataXY)
+	rownames(Theta)<- colnames(Theta) <- colnames(dataXY)	
+	
+	# Beta, Psi & Sigma matrices:
+	betas<- function(Theta){
+	 sapply(1:p, function(x) Theta[x,]/Theta[x,x]) * -1
+	}
+	Beta<- ifelse(dadj == 1, betas(Theta), 0)
+	Psi<- ifelse(dadj == 100, Sigma, 0)
+	diag(Psi)<- apply(scale(dataXY) %*% (diag(p)- Beta), 2, var)
+	df<- p*(p+1)/2 - (sum(Beta != 0) + (sum(Psi != 0)-p)/2 + p)
+	cat(paste0("GGM (constrained) solver ended normally after ", 0, " iterations"),"\n\n")
+	idx<- fitIndices(n, df, covXY, Sigma, Theta)
+	cat("deviance/df:", idx[1]/idx[2], " srmr:", round(idx[3],7), "\n\n")
+
+	est <- parameterEstimates.GGM(dadj, Beta, Psi, Theta, R=covXY, n=n)
 	if (!is.null(group)) {
-		adj <- cbind(c(0, rep(1, p - 1)), rbind(rep(1, p - 1), adj))
-		colnames(adj)[1] <- rownames(adj)[1] <- "group"
-		dadj <- cbind(rep(0, p), rbind(rep(1, p - 1), dadj))
-		colnames(dadj)[1] <- rownames(dadj)[1] <- "group"
-	}
-
-	# Constrained GGM
-	cggm <- GGMncv::constrained(covXY, adj)
-	Sigma <- cggm$Sigma
-	Theta <- cggm$Theta
-	rownames(Sigma) <- colnames(Sigma) <- colnames(dataXY)
-	rownames(Theta) <- colnames(Theta) <- colnames(dataXY)
-	fit <- list(Theta = Theta, n = n, R = covXY)
-	class(fit) <- c("ggmncv", "default")
-
-	# Beta, Psi & Sigma matrices
-	betas <- function(Theta) {
-		-1*sapply(1:p, function(x) Theta[x,]/Theta[x, x])
-	}
-	B <- ifelse(dadj == 1, betas(Theta), 0)
-	O <- ifelse(dadj == 100, t(diag(p) - B)%*%Sigma%*%(diag(p) - B), 0)
-	diag(O) <- apply(scale(dataXY) %*% (diag(p) - B), 2, var)
-	#Sigma <- solve(diag(p) - B)%*%O%*%t(solve(diag(p) - B))
-	cat(paste0("GGM (constrained) solver ended normally after ", 0,
-	           " iterations"), "\n\n")
-	df <- p*(p + 1)/2 - (sum(B != 0) + (sum(O != 0) - p)/2 + p)
-	idx <- fitIndices(n, df, covXY, Sigma, Theta)
-	cat("deviance/df:", idx[1]/idx[2], " srmr:", round(idx[3], 7), "\n\n")
-
-	# Edge pvalues based on the de-sparsified precision matrix
-	dggm <- suppressWarnings(
-		GGMncv::inference(fit, method = method, alpha = alpha))
-		#dggm <- GGMncv::inference(fit, method = method, alpha = alpha)
-		pvB <- ifelse(dadj == 1, dggm$pval_corrected, 0)
-		rownames(pvB) <- colnames(pvB) <- colnames(dataXY)
-		pvO <- ifelse(dadj == 100, dggm$pval_corrected, 0)
-		rownames(pvO) <- colnames(pvO) <- colnames(dataXY)
-
-	est <- parameterEstimates.GGM(object = list(B = B, O = O, pvB = pvB,
-	                              pvO = pvO))
-	if (!is.null(group)) {
-		gest <- est$Reg[est$Reg$rhs == "group",]
-		pval1 <- Brown.test(x = dataY, p = gest$pvalue, theta = gest$est,
-		                    tail = "positive")
-		pval2 <- Brown.test(x = dataY, p = gest$pvalue, theta = gest$est,
-		                    tail = "negative")
-		cat("Brown's combined P-value of node activation:", pval1, "\n\n")
-		cat("Brown's combined P-value of node inhibition:", pval2, "\n\n")
-	} else {
-		gest <- NULL
-	}
-
-	# Output objects
-	fit <- list(cggm = fit, Beta = B, Psi = O, fitIdx = idx,
-	            parameterEstimates = est)
-	ig <- colorGraph(est = est$Reg, graph = ig, group = group, alpha = 0.05)
-	#gplot(ig)
-	if (is.null(group)) dataXY <- cbind(group = rep(NA, n), dataXY)
-	class(fit) <- "GGM"
-
-	return(list(fit = fit, gest = gest, model = NULL, graph = ig,
-	            dataXY = dataXY))
+	 gest<- est[est$op == "~" & est$rhs == "group",]
+	 pval1<- Brown.test(x=dataY, p=gest$pvalue, theta=gest$est, tail="positive")
+	 pval2<- Brown.test(x=dataY, p=gest$pvalue, theta=gest$est, tail="negative")
+	 cat("Brown's combined P-value of node activation:", pval1, "\n\n")
+     cat("Brown's combined P-value of node inhibition:", pval2, "\n\n")
+	}else{gest<- NULL}
+	
+	# output objects:
+	Reg <- est[which(est$op == "~"),]
+	ig<- colorGraph(est=Reg, graph=ig, group=group, alpha=0.05)
+	if (is.null(group)) dataXY<- cbind(group=rep(NA, n), dataXY)
+	fit<- list(Theta=Theta, Beta=Beta, Psi=Psi, fitIdx=idx, parameterEstimates=est)
+	class(fit)<- "GGM"
+	
+	return( list(fit=fit, gest=gest, model=NULL, graph=ig, dataXY=dataXY) )	
 }
 
-SEMggm2 <- function(graph, data, group, method = "none", alpha = 0.05, ...)
+SEMggm2<- function(graph, data, group, method = "none", alpha = 0.05, ...) 
 {
-	# Set graph and data objects
-	nodes <- colnames(data)[colnames(data) %in% V(graph)$name]
-	ig <- induced_subgraph(graph, vids = which(V(graph)$name %in% nodes))
-	E(ig)$weight <- ifelse(which_mutual(ig), 100, 1)
-	dadj <- as_adj(ig, attr="weight", sparse=FALSE)[nodes,nodes]#dim(dadj)
-	dataY <- data[,nodes]
-	data1 <- dataY[group == 1,]
-	data0 <- dataY[group == 0,]
-	n1 <- nrow(data1)
-	n0 <- nrow(data0)
-	n <- n1 + n0
-	p <- ncol(dataY)
-
-	# Constrained GGM for group = 1
-	cggm1 <- quiet(SEMggm(graph, data1, group = NULL, method = method,
-	                      alpha = alpha))
-	fit1 <- cggm1$fit$cggm
-
-	# Constrained GGM for group = 0
-	cggm0 <- quiet(SEMggm(graph, data0, group = NULL, method = method,
-	                      alpha = alpha))
-	fit0 <- cggm0$fit$cggm
-
-	# Two-group fit indices
-	cat(paste0("GGM (constrained) solver ended normally after ", 0,
-	           " iterations"), "\n\n")
-	srmr <- (n1/n)*cggm1$fit$fitIdx[3] + (n0/n)*cggm0$fit$fitIdx[3]
-	dev <- cggm1$fit$fitIdx[1] + cggm0$fit$fitIdx[1]
-	df <- cggm1$fit$fitIdx[2] + cggm0$fit$fitIdx[2]
-	cat("deviance/df:", dev/df , " srmr:", round(srmr, 7), "\n\n")
-
-	# Edge differences based on the de-sparsified precision matrix
-	dggms<- suppressWarnings(
-		GGMncv::compare_edges(fit1, fit0, method = method, alpha = alpha))
-	#dggms <- GGMncv::compare_edges(fit1, fit0, method = method, alpha = alpha)
-	d_est <- cggm1$fit$Beta - cggm0$fit$Beta
-	d_pv <- ifelse(dadj == 1, dggms$pvals_corrected, 0)
-	rownames(d_pv) <- colnames(d_pv) <- colnames(dataY)
-
+	# Set graph and data objects:
+	nodes<- colnames(data)[colnames(data) %in% V(graph)$name]
+	ig<- induced_subgraph(graph, vids= which(V(graph)$name %in% nodes))
+	E(ig)$weight<- ifelse(which_mutual(ig), 100, 1)
+	dadj<- as_adj(ig, attr="weight", sparse=FALSE)[nodes,nodes]
+	dataY<- data[,nodes]
+	data1<- dataY[group == 1,]
+	data0<- dataY[group == 0,]
+	n1<- nrow(data1)
+	n0<- nrow(data0)
+	n<- n1 + n0
+	p<- ncol(dataY)
+		
+	# constrained GGM for group = 1
+	cggm1<- quiet(SEMggm(graph, data1, group=NULL, method = method, alpha = alpha))
+	est1<- parameterEstimates(cggm1$fit)
+	# constrained GGM for group = 0
+	cggm0<- quiet(SEMggm(graph, data0, group=NULL, method = method, alpha = alpha))
+	est0<- parameterEstimates(cggm0$fit)
+	# two-group fit indices
+	cat(paste0("GGM (constrained) solver ended normally after ", 0, " iterations"),"\n\n")
+	srmr<- (n1/n)*cggm1$fit$fitIdx[3] + (n0/n)*cggm0$fit$fitIdx[3]
+	dev<- cggm1$fit$fitIdx[1] + cggm0$fit$fitIdx[1]
+	df<- cggm1$fit$fitIdx[2] + cggm0$fit$fitIdx[2]
+	cat("deviance/df:", dev/df , " srmr:", round(srmr,7), "\n\n")
+		
+	# Edge differences based on the de-sparsified precision matrix	
+	est1<- est1[est1$op == "~",] #case
+	est0<- est0[est0$op == "~",] #control
+	d_omega<- est1$omega - est0$omega #case - control
+	d_est <- est1$est - est0$est
 	if (sum(d_est) != 0) {
-		dest <- parameterEstimates.GGM(object = list(B = d_est, O = diag(p),
-		                               pvB = d_pv, pvO = diag(p)),
-		                               dest = TRUE)[[1]]
-		pval1 <- Brown.test(x = dataY, p = dest$pvalue, theta = dest$d_est,
-		                    tail = "positive")
-		pval2 <- Brown.test(x = dataY, p = dest$pvalue, theta = dest$d_est,
-		                    tail = "negative")
-		cat("Brown's combined P-value of edge activation:", pval1, "\n\n")
-		cat("Brown's combined P-value of edge inhibition:", pval2, "\n\n")
-		ig <- colorGraph(est = dest, graph = ig, group = NULL, alpha = 0.05)
-	} else {
-		dest <- NULL
-	}
+	 d_se<- sqrt(est1$se^2 + est0$se^2)
+	 d_z<- d_omega/d_se
+	 pvalue<- 2*(1-pnorm(abs(d_z)))
+	 d_lower<- d_omega - 1.96*d_se
+	 d_upper<- d_omega + 1.96*d_se
+	 pval1<- Brown.test(x=dataY, p=pvalue, theta=d_est, tail="positive")
+	 pval2<- Brown.test(x=dataY, p=pvalue, theta=d_est, tail="negative")
+	 cat("Brown's combined P-value of edge activation:", pval1, "\n\n")
+     cat("Brown's combined P-value of edge inhibition:", pval2, "\n\n")
+	 dest<- cbind(est0[,1:3], d_est, d_omega, d_se, d_z, pvalue, d_lower, d_upper)
+	 class(dest)<- c("lavaan.data.frame" ,"data.frame")
+	 ig<- colorGraph(est=dest, graph=ig, group=NULL, alpha=0.05)
+	}else{ dest<- NULL }
 
-	# Output objects
-	est <- list(Group_1 = cggm1$fit[[5]], Group_0 = cggm0$fit[[5]])
-	fit <- list(Group_0 = cggm0$fit, Group_1 = cggm1$fit,
-	            parameterEstimates = est)
-	dataXY <- cbind(c(rep(1, n1), rep(0, n0)), rbind(data1, data0))
-
-	return(list(fit = fit, dest = dest, model = NULL, graph = ig,
-	            dataXY = dataXY))
+	# output objects :
+	fit<- list(Group_1 = cggm1$fit, Group_0 = cggm0$fit)
+	dataXY<- cbind(c(rep(1,n1),rep(0,n0)),rbind(data1,data0))
+		
+	return( list(fit=fit, dest=dest, model=NULL, graph=ig, dataXY=dataXY) )	
 }
 
-parameterEstimates.GGM <- function(object, dest = FALSE, ...)
+parameterEstimates.GGM <- function(dadj, Beta, Psi, Theta, R, n, ...)
 {
-	# Output of parameter estimates
-
-	B <- gdata::unmatrix(object$B, byrow = TRUE)
-	B <- B[which(B != 0)]
-
-	O <- ifelse(lower.tri(object$O, diag = TRUE), object$O, 0)
-	rownames(O) <- colnames(O) <- rownames(object$O)
-
-	O <- gdata::unmatrix(O, byrow = TRUE)
-	O <- O[which(O != 0)]
-	P <- c(B, O)
-	PvB <- gdata::unmatrix(object$pvB, byrow = TRUE)
-	PvB <- PvB[which(PvB != 0)]
-
-	PvO <- ifelse(lower.tri(object$pvO, diag = TRUE), object$pvO, 0)
-	rownames(PvO) <- colnames(PvO) <- rownames(object$pvO)
-
-	diag(PvO) <- ifelse(diag(PvO) == 0, 1E-9, diag(PvO))
-	PvO <- gdata::unmatrix(PvO, byrow = TRUE)
-	PvO <- PvO[which(PvO != 0)]
-	Pv <- c(PvB, PvO)
-	Pv <- Pv[names(P)]
-
-	est <- NULL
+	# Convert into a vector the output of parameter estimates	
+	A <- gdata::unmatrix(dadj, byrow=TRUE)
+	B <- gdata::unmatrix(Beta, byrow=TRUE)
+	B <- na.omit(ifelse(A == 1, B, NA))
+	O <- ifelse(lower.tri(Psi, diag=TRUE), Psi, 0)
+	rownames(O)<-colnames(O)<- rownames(Psi)
+    O <- gdata::unmatrix(O, byrow=TRUE)
+	O <- O[-which(O == 0)]
+	
+	L <- NULL
+	P <- c(B,O)
 	for(j in 1:length(P)) {
-		s <- strsplit(names(P)[j], ":")
-		lhs <- s[[1]][2]
-		rhs <- s[[1]][1]
-		if (j <= length(B)) {
-			est <- rbind(est, data.frame(lhs, op = "~", rhs, est = P[j],
-			             pvalue = Pv[j],
-			             stringsAsFactors = FALSE))
-		} else {
-			est <- rbind(est, data.frame(lhs, op = "~~", rhs, est = P[j],
-			             pvalue = Pv[j],
-			             stringsAsFactors = FALSE))
-		}
+	 s<- strsplit(names(P)[j],":")
+	 lhs<- s[[1]][2]
+	 rhs<- s[[1]][1]
+	 if(j <= length(B)){ 
+	  L<- rbind(L, data.frame(lhs, op= "~", rhs))
+	 }else{
+	  L<- rbind(L, data.frame(lhs, op= "~~", rhs))}
 	}
+	
+	# Equation 7 in
+	# Jankova, J., & Van De Geer, S. (2015). Confidence intervals for high-dimensional
+	# inverse covariance estimation. Electronic Journal of Statistics, 9(1), 1205-1229.
+	Tstar <- 2 * Theta - Theta %*% R %*% Theta
+	Tstar <- gdata::unmatrix(Tstar, byrow=TRUE)
+	Bstar <- Tstar[names(Tstar) %in% names(B)]
+	Ostar <- Tstar[names(Tstar) %in% names(O)]
+	# standard error
+	sds <- sqrt((tcrossprod(diag(Theta)) + Theta^2))
+	sds <- gdata::unmatrix(sds, byrow=TRUE)
+	Bse <- sds[names(sds) %in% names(B)]/sqrt(n)
+	Ose <- sds[names(sds) %in% names(O)]/sqrt(n)
+	# z test
+	z <- c(Bstar,Ostar)/c(Bse,Ose)
 
+	est <- data.frame(L, est = c(B,O),
+	 omega = c(Bstar,Ostar), se = c(Bse,Ose), z = z,
+	 pvalue = 2*(1-pnorm(abs(z))),
+	 ci.lower = (c(Bstar,Ostar) - 1.96*c(Bse,Ose)),
+	 ci.upper = (c(Bstar,Ostar) + 1.96*c(Bse,Ose)))
 	rownames(est) <- NULL
-	reg <- est[which(est$op == "~"),]
-	cov <- est[which(est$op == "~~"),]
-	sel <- which(cov$lhs == cov$rhs)
-	var <- cov[sel,]
-	cov <- cov[-sel,]
-	if (dest) colnames(reg)[4] <- "d_est"
+	#class(est) <- c("lavaan.data.frame" ,"data.frame")
 
-	return(list(Reg = reg, Cov = cov, Var = var))
-}
-
-#' @title GGM model summary
-#'
-#' @description Generate a summary for a constrained Gaussian Graphical
-#' Model (GGM) similar to lavaan-formated summary
-#'
-#' @param object A constrained GGM fitted model object.
-#' @param ... Currently ignored.
-#'
-#' @method summary GGM
-#' @return Shown the lavaan-formatted summary to console
-#'
-#' @author Mario Grassi \email{mario.grassi@unipv.it}
-#'
-#' @seealso \code{\link[SEMgraph]{SEMrun}}.
-#'
-#' @examples
-#' sem0 <- SEMrun(sachs$graph, log(sachs$pkc), algo = "cggm")
-#' summary(sem0$fit)
-#'
-#' @export
-summary.GGM <- function(object, ...)
-{
-	.local <- function(object) {
-		it <- 0
-		t <- object$fitIdx[6]
-		n <- object$fitIdx[5]
-		dev <- round(object$fitIdx[1], 3)
-		df <- object$fitIdx[2]
-		srmr <- round(object$fitIdx[3], 3)
-
-		cat(paste0("GGM (constrained) solver ended normally after ", it,
-		           " iterations"), "\n\n")
-		cat(paste0("  Estimator                                       ML"),
-		           "\n")
-		cat(paste0("  Optimization method                             CGGM"),
-		           "\n\n")
-		cat(paste0("  Number of free parameters                       ",
-		           t), "\n\n")
-		cat(paste0("  Number of observations                          ",
-		           n), "\n")
-		cat("\nModel Test User Model\n\n")
-		cat(paste0("  Test statistic (Deviance)                       ",
-		           dev), "\n")
-		cat(paste0("  Degrees of freedom (df)                         ",
-		           df), "\n")
-		cat(paste0("  Deviance/df                                     ",
-		           round(dev/df, 3)), "\n")
-		cat(paste0("  Standardized Root Mean Square Residual (srmr)   ",
-		           srmr), "\n")
-		cat("\nParameter Estimates:\n\n")
-
-		#print(object$parameterEstimates)
-		K <- c("Regressions:", "Covariances:", "Variances:")
-		L <- object$parameterEstimates
-		for (l in 1:3) {
-			cat(K[l], "\n\n")
-			print(data.frame(lapply(L[[l]], function(y) {
-					if(is.numeric(y)) round(y, 3) else y
-				}
-			)))
-			cat("\n")
-		}
-	}
-	.local(object)
+	return(est)
 }
 
 fitIndices <- function(n, df, S, Sigma, Theta = NULL, ...)
@@ -1290,3 +1160,194 @@ msep.test<- function(bap, S, n, verbose, ...)
 	
 	return( SET=na.omit(SET) )
 }
+
+#' @title Parameter Estimates of a fitted SEM
+#'
+#' @description Wrapper of the lavaan parameterEstimates() function
+#' for RICF and CGGM algorithms
+#'
+#' @param fit A RICF or constrained GGM fitted model object.
+#' @param ... Currently ignored.
+#'
+#' @return A data.frame containing the estimated parameters
+#'
+#' @author Mario Grassi \email{mario.grassi@unipv.it}
+#'
+#' @examples
+#' ricf1 <- SEMrun(sachs$graph, log(sachs$pkc), sachs$group, algo = "ricf")
+#' parameterEstimates(ricf1$fit)
+#'
+#' cggm1 <- SEMrun(sachs$graph, log(sachs$pkc), sachs$group, algo = "cggm")
+#' parameterEstimates(cggm1$fit)
+#'
+#' @export
+parameterEstimates<- function(fit, ...)
+{
+	if (inherits(fit, "lavaan")){
+	 est <- lavaan::parameterEstimates(fit)
+	 est$lhs <- gsub("z", "", est$lhs)
+	 est$rhs <- gsub("z", "", est$rhs)
+	 if ("group" %in% colnames(est)){
+	  est <- cbind(est[,-c(4,5)], group=est[,5])
+	 }
+	}
+	else if (!inherits(fit, "lavaan")){
+	 if (length(fit) != 2){
+	  est0<- fit$parameterEstimates
+	  sel<- which(est0$lhs == "group")
+	  if(length(sel) != 1){
+	   est <- est0
+	  }else{
+	   est <- est0[-sel,]
+	   est <- rbind(est, est0[sel,])
+	  }
+	 }else{
+	  est0 <- fit$Group_0$parameterEstimates
+	  est1 <- fit$Group_1$parameterEstimates
+	  group <- c(rep(1, nrow(est0)), rep(2,nrow(est1)))
+	  est <- cbind(rbind(est0,est1),group)
+	 } 
+	}
+	class(est)<- c("lavaan.data.frame" ,"data.frame")
+	return(est)
+}
+
+#' @title RICF model summary
+#'
+#' @description Generate a summary for a RICF solver similar to
+#' lavaan-formatted summary
+#'
+#' @param object A RICF fitted model object.
+#' @param ... Currently ignored.
+#'
+#' @method summary RICF
+#' @return Shown the lavaan-formatted summary to console
+#'
+#' @author Mario Grassi \email{mario.grassi@unipv.it}
+#'
+#' @seealso \code{\link[SEMgraph]{SEMrun}}.
+#'
+#' @examples
+#' sem1 <- SEMrun(sachs$graph, log(sachs$pkc), sachs$group, algo = "ricf")
+#' summary(sem1$fit)
+#'
+#' @export
+summary.RICF <- function(object, ...)
+{
+	.local <- function(object) {
+		it <- object$ricf$it
+		t <- object$fitIdx[6]
+		n <- object$fitIdx[5]
+		dev <- round(object$fitIdx[1], 3)
+		df <- object$fitIdx[2]
+		srmr <- round(object$fitIdx[3], 3)
+
+		cat(paste0("RICF solver ended normally after ", it, " iterations"),
+		           "\n\n")
+		cat(paste0("  Estimator                                       ML"),
+		           "\n")
+		cat(paste0("  Optimization method                             RICF"),
+		           "\n\n")
+		cat(paste0("  Number of free parameters                       ",
+		           t), "\n\n")
+		cat(paste0("  Number of observations                          ",
+		           n), "\n")
+		cat("\nModel Test User Model\n\n")
+		cat(paste0("  Test statistic (Deviance)                       ",
+		           dev), "\n")
+		cat(paste0("  Degrees of freedom (df)                         ",
+		           df), "\n")
+		cat(paste0("  Deviance/df                                     ",
+		           round(dev/df, 3)), "\n")
+		cat(paste0("  Standardized Root Mean Square Residual (srmr)   ",
+		           srmr), "\n")
+		cat("\nParameter Estimates:\n\n")
+
+		#print(fit$parameterEstimates)
+		K <- c("Regressions:", "Covariances:", "Variances:")
+		L <- object$parameterEstimates
+		reg <- L[which(L$op == "~"),]
+		cov <- L[which(L$op == "~~"),]
+		sel <- which(cov$lhs == cov$rhs)
+		L <- list(reg, cov[-sel,], cov[sel,])
+		for (l in 1:3) {
+			cat(K[l], "\n\n")
+			print(data.frame(lapply(L[[l]], function(y) {
+					if(is.numeric(y)) round(y, 3) else y
+				}
+			)))
+			cat("\n")
+		}
+	}
+	.local(object)
+}
+
+#' @title GGM model summary
+#'
+#' @description Generate a summary for a constrained Gaussian Graphical
+#' Model (GGM) similar to lavaan-formated summary
+#'
+#' @param object A constrained GGM fitted model object.
+#' @param ... Currently ignored.
+#'
+#' @method summary GGM
+#' @return Shown the lavaan-formatted summary to console
+#'
+#' @author Mario Grassi \email{mario.grassi@unipv.it}
+#'
+#' @seealso \code{\link[SEMgraph]{SEMrun}}.
+#'
+#' @examples
+#' sem0 <- SEMrun(sachs$graph, log(sachs$pkc), algo = "cggm")
+#' summary(sem0$fit)
+#'
+#' @export
+summary.GGM <- function(object, ...)
+{
+	.local <- function(object) {
+		it <- 0
+		t <- object$fitIdx[6]
+		n <- object$fitIdx[5]
+		dev <- round(object$fitIdx[1], 3)
+		df <- object$fitIdx[2]
+		srmr <- round(object$fitIdx[3], 3)
+
+		cat(paste0("GGM (constrained) solver ended normally after ", it,
+		           " iterations"), "\n\n")
+		cat(paste0("  Estimator                                       ML"),
+		           "\n")
+		cat(paste0("  Optimization method                             CGGM"),
+		           "\n\n")
+		cat(paste0("  Number of free parameters                       ",
+		           t), "\n\n")
+		cat(paste0("  Number of observations                          ",
+		           n), "\n")
+		cat("\nModel Test User Model\n\n")
+		cat(paste0("  Test statistic (Deviance)                       ",
+		           dev), "\n")
+		cat(paste0("  Degrees of freedom (df)                         ",
+		           df), "\n")
+		cat(paste0("  Deviance/df                                     ",
+		           round(dev/df, 3)), "\n")
+		cat(paste0("  Standardized Root Mean Square Residual (srmr)   ",
+		           srmr), "\n")
+		cat("\nParameter Estimates:\n\n")
+
+		#print(object$parameterEstimates)
+		K <- c("Regressions:", "Covariances:", "Variances:")
+		L <- object$parameterEstimates
+		reg <- L[which(L$op == "~"),]
+		cov <- L[which(L$op == "~~"),]
+		sel <- which(cov$lhs == cov$rhs)
+		L <- list(reg, cov[-sel,], cov[sel,])
+		for (l in 1:3) {
+			cat(K[l], "\n\n")
+			print(data.frame(lapply(L[[l]], function(y) {
+					if(is.numeric(y)) round(y, 3) else y
+				}
+			)))
+			cat("\n")
+		}
+	}
+	.local(object)
+}
